const DATA_URL = new URL("/brochures/", document.baseURI)
const PARSER = new DOMParser()
const SERIALIZER = new XMLSerializer()
const DATE_FORMATTER = new Intl.DateTimeFormat("fr", {dateStyle: "short"})

const FEATURES = [
    function titleFeature(doc, data){
        doc.title = data.get("title") + " " + doc.title
    },
    
    function nameFeature(doc, data) {
        doc.querySelector('[property="name"]').textContent = data.get("title")
    },

    function hrefFeature(doc, data) {
        doc.querySelector('a[href="original.pdf"]').href = "./"+data.get("original-filename")
    },

    function uploadDateFeature(doc, data) {
        let el = doc.querySelector('[property="uploadDate"]')
        el.textContent = DATE_FORMATTER.format(new Date(data.get("upload-date")))
        el.setAttribute("datetime", data.get("upload-date"));
        el.setAttribute("content", data.get("upload-date"));
    },

    function authorFeature(doc, data){
        if(!data.has("author"))
            return

        let el = doc.createElement("p")
        el.append(doc.createTextNode("Par "))

        let authorEl = doc.createElement("span")
        authorEl.textContent = data.get("author")
        authorEl.setAttribute("property", "author")
        el.append(authorEl)

        doc.querySelector("h1").insertAdjacentElement("afterend", el)
    },

    function abstractFeature(doc, data){
        if(!data.has("abstract"))
            return

        let el = doc.createElement("p")
        el.textContent = data.get("abstract")
        el.setAttribute("property", "abstract")

        doc.querySelector("h2").insertAdjacentElement("beforebegin", el)
    }
]

export async function uploadDocument(documentFile, documentDetails = new FormData()){

    let uniqueName = await getUniqueName(documentDetails)

    let baseURI = new URL(`./${uniqueName}/`, DATA_URL)

    let originalFileName = `${slugify(documentDetails.get("title"))}-original.pdf`
    documentDetails.set("original-filename", originalFileName)

    let uploadProgress = fetch(new URL(`./${originalFileName}`, baseURI), {
            method: "PUT",
            headers: {
                "Content-Type": "application/pdf"
            },
            body: documentFile
        })

    let page = PARSER.parseFromString(await (await fetch("/brochure.template.html")).text(), "text/html")

    for(let feature of FEATURES){
        feature(page, documentDetails)
    }

    let responses = await Promise.all([
        fetch(new URL("./index.html", baseURI), {
            method: "PUT",
            headers: {
                "Content-Type": "text/html"
            },
            body: SERIALIZER.serializeToString(page)
        }),
        uploadProgress
    ]);

    for(let res of responses) {
        if(!res.ok){
            throw new Error(`Network error ${res.status} ${res.statusText}`)
        }
    }

    return baseURI.toString()
}

async function getUniqueName(documentDetails){
    let nameCandidate, isUnique;

    do {
        nameCandidate = getRandomName(documentDetails)

        let res = await fetch(new URL(`./${nameCandidate}/index.html`, DATA_URL), {
            method: "HEAD"
        });

        if(!res.ok && res.status != 404){
            throw new Error(`Network error ${res.status} ${res.statusText}`)
        }

        isUnique = res.status == 404

    } while(!isUnique)


    return nameCandidate
}

function getRandomName(documentDetails){
    let source = new Uint8Array(16)
    crypto.getRandomValues(source)
    let parts = [toHex(source)]

    if(documentDetails.get("author")){
        parts.push(slugify(documentDetails.get("author")))
    }

    if(documentDetails.get("title")){
        parts.push(slugify(documentDetails.get("title")))
    }

    return parts.join("-")
}

function toHex(buffer) {
    return Array.prototype.map.call(buffer, x => x.toString(16).padStart(2, '0')).join('');
}

function slugify(str){
    return str.toLowerCase().replace(/\//g,"-").replace(/\s/g, "-").replace(/--+/, "-")
}